/* -*-c-*-
 *
 * $Id: bres.h,v 1.1 1999/07/01 08:56:23 mdw Exp $
 *
 * Background reverse name resolution
 *
 * (c) 1999 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: bres.h,v $
 * Revision 1.1  1999/07/01 08:56:23  mdw
 * Initial revision
 *
 */

#ifndef RES_H
#define RES_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <mLib/sel.h>
#include <mLib/selbuf.h>

/*----- Data structures ---------------------------------------------------*/

/* --- Client allocated request block --- */

typedef struct bres_client {
  struct bres_client *next, *prev;	/* Queue of waiting resolve jobs */
  struct bres_server *rs;		/* Pointer to attached server */
  struct in_addr addr;			/* Address to be resolved */
  void (*func)(const char */*host*/, void */*p*/); /* Handler function */
  void *p;				/* Argument for handler function */
} bres_client;

/* --- Server maintained resolver blocks --- */

typedef struct bres_server {
  struct bres_server *next, *prev;	/* Doubly-linked list of servers */
  pid_t kid;				/* Process id of server process */
  selbuf b;				/* Input line-buffer selector */
  sel_timer t;				/* Timeout for idle servers */
  int fd;				/* File descriptor for writing */
  struct bres_client *rc;		/* Pointer to attached client */
} bres_server;

/*----- Functions provided ------------------------------------------------*/

/* --- @bres_abort@ --- *
 *
 * Arguments:	@bres_client *rc@ = pointer to client block
 *
 * Returns:	---
 *
 * Use:		Removes a queued job.
 */

extern void bres_abort(bres_client */*rc*/);

/* --- @bres_resolve@ --- *
 *
 * Arguments:	@bres_client *rc@ = pointer to client block
 *		@struct in_addr addr@ = address to resolve
 *		@void (*func)(const char *host, void *p)@ = handler function
 *		@void *p@ = argument for handler function
 *
 * Returns:	---
 *
 * Use:		Adds a resolver job to the queue.  The job will be processed
 *		when there's a spare resolver process to deal with it.
 */

extern void bres_resolve(bres_client */*rc*/, struct in_addr /*addr*/,
			void (*/*func*/)(const char */*host*/, void */*p*/),
			void */*p*/);

/* --- @bres_init@ --- *
 *
 * Arguments:	@sel_state *s@ = pointer to select multiplexor
 *
 * Returns:	---
 *
 * Use:		Initializes the background resolver for use.
 */

extern void bres_init(sel_state */*s*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
